/*
 * SonarQube JavaScript Plugin
 * Copyright (C) 2011-2016 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package org.sonar.javascript.cfg;

import com.google.common.collect.ImmutableSet;
import java.util.Map;
import java.util.Set;
import org.sonar.plugins.javascript.api.tree.Tree;

/**
 * The end node of a {@link ControlFlowGraph} being built.
 */
class EndBlock extends MutableBlock {

  @Override
  public void addElement(Tree element) {
    throw new UnsupportedOperationException("Cannot add element to end node");
  }

  @Override
  public Set<MutableBlock> successors() {
    return ImmutableSet.of();
  }

  @Override
  public void replaceSuccessors(Map<MutableBlock, MutableBlock> replacements) {
    // Nothing to replace
  }

  @Override
  public MutableBlock trueSuccessor() {
    throw new UnsupportedOperationException("End node has no successor");
  }

  @Override
  public MutableBlock falseSuccessor() {
    throw new UnsupportedOperationException("End node has no successor");
  }

}
